package src.hw8convex;

public class SimpleModelConvex implements MathFunction {
	private double[] x;
	private double[] y;
	private int n;		// Number of unknowns
	private double[] upBound;
	private double[] lowBound;
	
	public SimpleModelConvex() {
    	n= 2;
    	x= new double[n];
		upBound= new double[n];
		upBound[0]= 4.0;
		upBound[1]= 6.0;
		lowBound= new double[n];		// Lower bounds are zero
		y= new double[n];
	}
	
    public double func(double[] x) {
    	return x[0]*x[0]+2.0*x[1]*x[1]-2.0*x[0]*x[1]-10.0*x[1];
    }
    
    public double[] gradient() {
    	int n= x.length;
    	double[] d= new double[n];
    	d[0]= 2.0*x[0] - 2.0*x[1];
    	d[1]= 4.0*x[1] - 2.0*x[0]- 10.0;
    	return d;
    }
    
    public double derivative(double alpha) {
    	double d1= (x[0]-x[1])*(y[0]-x[0])+ (y[1]-x[1])*(2*x[1]-x[0]-5);
    	double d2= 2*(y[0]-x[0])*(y[1]-x[1])- 2*(y[1]-x[1])*(y[1]-x[1])- (y[0]-x[0])*(y[0]-x[0]);
    	double deriv = d1 - alpha*d2;
    	return deriv;
    }
    
    private double lineSearch(MathFunction d, double a, double b) {
    	final double TOLERANCE= 1E-8;		// Square root of machine precision
    	final double MAX_ITERATIONS= 1000;
        double m;							// Midpoint
        int counter= 0;
        for (m= (a+b)/2.0; Math.abs(a-b) > TOLERANCE; m= (a+b)/2.0) {
        	counter++;
            if  (d.derivative(m)< 0.0)    // If derivative negative, 
                a= m;       // Use left subinterval
            else
                b= m;       // Use right subinterval
            if (counter > MAX_ITERATIONS)
            	break;
        }
        return m;
    }
    
    private double[] linearProgram(double[] lpCoeff) {	// Minimization
    	for (int i= 0; i < n; i++) {
    		if (lpCoeff[i]> 0 )
    			y[i]= lowBound[i];
    		else 
    			y[i]= upBound[i];
    	}
    	return y;
    }
    
    private void printDirection(double a) {
    	System.out.print("Direction  y: \t");
       	for (int i= 0; i < n; i++)
    		System.out.printf("%5.2f ",y[i]);
       	System.out.printf("               %5.3f", a);
    	System.out.println();	
    }  
    
    private double move(double alpha) {
    	double sumX= 0.0;
    	double sumRootMeanDiffX= 0.0;
    	for (int i= 0; i < n; i++) {
    		double variableChange= alpha*(y[i] - x[i]);
    		x[i]+= variableChange;
    		sumX+= x[i];
    		sumRootMeanDiffX+= variableChange*variableChange;
    	}
    	// Compute convergence criterion here, since we have current and previous x
    	return Math.sqrt(sumRootMeanDiffX)/sumX;
    }
    
    private void printMove() {
    	System.out.print("Move       x: \t");
       	for (int i= 0; i < n; i++)
    		System.out.printf("%5.2f ",x[i]);
       	System.out.printf("   %6.3f ", func(x));
    	System.out.println("\n");	
    }  
    
    public void maximize() {
    	// Initialization in constructor
    	final double CRITERION= 0.001;		// Convergence criterion: 0.1% change in x (root mean square)
    	final int MAX_ITERATIONS= 100;		// Should be set higher in real code
        double convergence;
        int iterations= 0;
        printMove();
        do {
        	double[] lpCoeff= gradient();
        	y= linearProgram(lpCoeff);		// Constraints are instance variables, never change
        	double alpha= lineSearch(this, 0.0, 1.0);	// Alpha between 0 and 1
        	printDirection(alpha);
        	convergence= move(alpha);
        	printMove();
        	// Step 5: Check convergence
        	iterations++;
        } while (convergence > CRITERION && iterations < MAX_ITERATIONS);
    }

    public static void main(String[] args) {
        SimpleModelConvex g= new SimpleModelConvex();
        g.maximize();
    }   
}